package flowserver

import (
	"fmt"
	"log"
	"net"
	"net/http"
	"net/http/httputil"
	"net/url"

	"github.com/gohxs/prettylog"
	"github.com/gohxs/webu"
	"github.com/gohxs/webu/chain"
)

//go:generate go get dev.hexasoftware.com/hxs/genversion
//go:generate genversion -package flowserver -out version.go

// FlowServer structure
type FlowServer struct{}

//ListenAndServe starts the httpserver
func (f *FlowServer) ListenAndServe() error {

	fsm := FlowSessionManager{}

	// HttpPart
	c := chain.New(webu.ChainLogger(prettylog.New("req")))
	mux := http.NewServeMux()
	mux.Handle("/conn", c.Build(fsm.ServeHTTP))

	proxyURL, err := url.Parse("http://localhost:8081")
	if err != nil {
		return err
	}

	//mux.Handle("/", c.Build(webu.StaticHandler("web", "index.html")))
	mux.Handle("/", c.Build(httputil.NewSingleHostReverseProxy(proxyURL).ServeHTTP))

	port := 4000
	for {
		addr := fmt.Sprintf(":%d", port)
		s, err := net.Listen("tcp", addr)
		if err != nil {
			log.Println("Listen error:", err)
			port++
			continue
		}
		log.Println("Listening at:", addr)

		err = http.Serve(s, mux)
		if err != nil {
			log.Fatal(err)
		}
	}
}
